@@ -1,6 +1,5 @@
 /* enchant
  * Copyright (C) 2003, 2004 Dom Lachowicz
- * Copyright (C) 2016-2021 Reuben Thomas <rrt@sc3d.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -65,6 +64,7 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include "enchant-provider.h"
+#include "unused-parameter.h"
 
 #include "pwl.h"
 
@@ -75,7 +75,7 @@ static const gunichar BOM = 0xfeff;
 
 /*  A PWL dictionary is stored as a Trie-like data structure EnchantTrie.
  *  The EnchantTrie datatype is completely recursive - all child nodes
- *  are simply EnchantTrie pointers.  This means that all functions
+ *  are simply EnchantTrie pointers.  This means that all functions 
  *  that potentially modify a trie need to return the modified trie,
  *  as additional memory may have been allocated.
  *
@@ -202,7 +202,7 @@ EnchantPWL* enchant_pwl_init(void)
  *
  * Returns: a new PWL object used to store/check/suggest words
  * or NULL if the file cannot be opened or created
- */
+ */ 
 EnchantPWL* enchant_pwl_init_with_file(const char * file)
 {
 	g_return_val_if_fail (file != NULL, NULL);
@@ -233,13 +233,13 @@ static void enchant_pwl_refresh_from_file(EnchantPWL* pwl)
 	pwl->words_in_trie = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
 	FILE *f = g_fopen(pwl->filename, "r");
-	if (!f)
+	if (!f) 
 		return;
 
 	pwl->file_changed = stats.st_mtime;
 
 	enchant_lock_file (f);
-
+	
 	char buffer[BUFSIZ + 1];
 	size_t line_number = 1;
 	for (; NULL != (fgets (buffer, sizeof (buffer), f)); ++line_number)
@@ -248,12 +248,12 @@ static void enchant_pwl_refresh_from_file(EnchantPWL* pwl)
 			if(line_number == 1 && BOM == g_utf8_get_char(line))
 				line = g_utf8_next_char(line);
 
-			if(line[strlen(line)-1] != '\n' && !feof(f)) /* ignore lines longer than BUFSIZ. */
+			if(line[strlen(line)-1] != '\n' && !feof(f)) /* ignore lines longer than BUFSIZ. */ 
 				{
 					g_warning ("Line too long (ignored) in %s at line:%zu\n", pwl->filename, line_number);
 					while (NULL != (fgets (buffer, sizeof (buffer), f)))
 						{
-							if (line[strlen(buffer)-1]=='\n')
+							if (line[strlen(buffer)-1]=='\n') 
 								break;
 						}
 					continue;
@@ -268,7 +268,7 @@ static void enchant_pwl_refresh_from_file(EnchantPWL* pwl)
 						g_warning ("Bad UTF-8 sequence in %s at line:%zu\n", pwl->filename, line_number);
 				}
 		}
-
+	
 	enchant_unlock_file (f);
 	fclose (f);
 }
@@ -289,7 +289,7 @@ static void enchant_pwl_add_to_trie(EnchantPWL *pwl,
 		g_free (normalized_word);
 		return;
 	}
-
+	
 	g_hash_table_insert (pwl->words_in_trie, normalized_word, g_strndup(word,len));
 
 	pwl->trie = enchant_trie_insert(pwl->trie, normalized_word);
@@ -308,16 +308,13 @@ static void enchant_pwl_remove_from_trie(EnchantPWL *pwl,
 				pwl->trie = NULL; /* make trie empty if has no content */
 			}
 		}
-
+	
 	g_free(normalized_word);
 }
 
 void enchant_pwl_add(EnchantPWL *pwl,
-			 const char *const word, ssize_t len)
+			 const char *const word, size_t len)
 {
-	if (len < 0)
-		len = strlen (word);
-
 	enchant_pwl_refresh_from_file(pwl);
 
 	enchant_pwl_add_to_trie(pwl, word, len);
@@ -345,20 +342,19 @@ void enchant_pwl_add(EnchantPWL *pwl,
 							putc ('\n', f);
 					}
 
-				if (fwrite (word, sizeof(char), len, f) == (size_t)len)
-					putc ('\n', f);
+				if (fwrite (word, sizeof(char), len, f) == len)
+					{
+						putc ('\n', f);
+					}
 				enchant_unlock_file (f);
 				fclose (f);
-			}
+			}	
 	}
 }
 
 void enchant_pwl_remove(EnchantPWL *pwl,
-			 const char *const word, ssize_t len)
+			 const char *const word, size_t len)
 {
-	if (len < 0)
-		len = strlen (word);
-
 	if(enchant_pwl_check(pwl, word, len) == 1)
 		return;
 
@@ -399,7 +395,7 @@ void enchant_pwl_remove(EnchantPWL *pwl,
 									fwrite (searchstart, sizeof(char), length - (searchstart - contents), f);
 									break;
 								}
-							else
+							else 
 								{
 									char* foundend = needle+len;
 									if((needle == filestart || contents[needle-contents-1] == '\n' || contents[needle-contents-1] == '\r') &&
@@ -417,7 +413,7 @@ void enchant_pwl_remove(EnchantPWL *pwl,
 								}
 						}
 					g_free(key);
-
+					
 					GStatBuf stats;
 					if(g_stat(pwl->filename, &stats)==0)
 						pwl->file_changed = stats.st_mtime;
@@ -425,7 +421,7 @@ void enchant_pwl_remove(EnchantPWL *pwl,
 					enchant_unlock_file (f);
 
 					fclose (f);
-				}
+				}	
 			g_free(contents);
 		}
 }
@@ -465,7 +461,7 @@ static int enchant_is_all_caps(const char*const word, size_t len)
 				case G_UNICODE_SURROGATE:
 				case G_UNICODE_MODIFIER_LETTER:
 				case G_UNICODE_OTHER_LETTER:
-				case G_UNICODE_SPACING_MARK:
+				case G_UNICODE_COMBINING_MARK:
 				case G_UNICODE_ENCLOSING_MARK:
 				case G_UNICODE_NON_SPACING_MARK:
 				case G_UNICODE_DECIMAL_NUMBER:
@@ -493,7 +489,7 @@ static int enchant_is_all_caps(const char*const word, size_t len)
 	return hasCap;
 }
 
-static G_GNUC_PURE int enchant_is_title_case(const char * const word, size_t len)
+static _GL_ATTRIBUTE_PURE int enchant_is_title_case(const char * const word, size_t len)
 {
 	g_return_val_if_fail (word && *word, 0);
 
@@ -502,7 +498,7 @@ static G_GNUC_PURE int enchant_is_title_case(const char * const word, size_t len
 	if ((type != G_UNICODE_UPPERCASE_LETTER && type != G_UNICODE_TITLECASE_LETTER) ||
 		ch != g_unichar_totitle(ch))
 		return 0;
-
+			
 	for (const char* it = g_utf8_next_char(word); it < word + len; it = g_utf8_next_char(it))
 		{
 			type = g_unichar_type(g_utf8_get_char(it));
@@ -529,15 +525,12 @@ static gchar* enchant_utf8_strtitle(const gchar*str, gssize len)
 	return result;
 }
 
-int enchant_pwl_check(EnchantPWL *pwl, const char *const word, ssize_t len)
+int enchant_pwl_check(EnchantPWL *pwl, const char *const word, size_t len)
 {
-	if (len < 0)
-		len = strlen (word);
-
 	enchant_pwl_refresh_from_file(pwl);
 
 	int exists = enchant_pwl_contains(pwl, word, len);
-
+	
 	if(exists)
 		return 0;
 
@@ -570,8 +563,8 @@ static void enchant_pwl_check_cb(char* match,EnchantTrieMatcher* matcher)
 	(*((int*)(matcher->cbdata)))++;
 }
 
-static void enchant_pwl_case_and_denormalize_suggestions(EnchantPWL *pwl,
-							 const char *const word, size_t len,
+static void enchant_pwl_case_and_denormalize_suggestions(EnchantPWL *pwl, 
+							 const char *const word, size_t len, 
 							 EnchantSuggList* suggs_list)
 {
 	gchar* (*utf8_case_convert_function)(const gchar*str, gssize len) = NULL;
@@ -579,7 +572,7 @@ static void enchant_pwl_case_and_denormalize_suggestions(EnchantPWL *pwl,
 		utf8_case_convert_function = enchant_utf8_strtitle;
 	else if (enchant_is_all_caps(word, len))
 		utf8_case_convert_function = g_utf8_strup;
-
+	
 	for (size_t i = 0; i < suggs_list->n_suggs; ++i)
 		{
 			gchar* suggestion = g_hash_table_lookup (pwl->words_in_trie, suggs_list->suggs[i]);
@@ -590,7 +583,7 @@ static void enchant_pwl_case_and_denormalize_suggestions(EnchantPWL *pwl,
 				cased_suggestion = utf8_case_convert_function(suggestion, suggestion_len);
 			else
 				cased_suggestion = g_strndup(suggestion, suggestion_len);
-
+			
 			g_free(suggs_list->suggs[i]);
 			suggs_list->suggs[i] = cased_suggestion;
 		}
@@ -613,14 +606,11 @@ static int best_distance(char** suggs, const char *const word, size_t len)
 	return best_dist;
 }
 
-/* gives the best set of suggestions from pwl that are at least as good as the
+/* gives the best set of suggestions from pwl that are at least as good as the 
  * given suggs (if suggs == NULL just best from pwl) */
 char** enchant_pwl_suggest(EnchantPWL *pwl, const char *const word,
-			   ssize_t len, char** suggs, size_t* out_n_suggs)
+			   size_t len, char** suggs, size_t* out_n_suggs)
 {
-	if (len < 0)
-		len = strlen (word);
-
 	int max_dist = suggs ? best_distance(suggs, word, len) : ENCHANT_PWL_MAX_ERRORS;
 	max_dist = MIN (max_dist, ENCHANT_PWL_MAX_ERRORS);
 
@@ -643,7 +633,7 @@ char** enchant_pwl_suggest(EnchantPWL *pwl, const char *const word,
 	(*out_n_suggs) = sugg_list.n_suggs;
 
 	enchant_pwl_case_and_denormalize_suggestions(pwl, word, len, &sugg_list);
-
+	
 	return sugg_list.suggs;
 }
 
@@ -661,8 +651,9 @@ static void enchant_pwl_suggest_cb(char* match,EnchantTrieMatcher* matcher)
 	size_t loc;
 	for(loc=0; loc < sugg_list->n_suggs; loc++) {
 		/* Better than an existing suggestion, so stop */
-		if(sugg_list->sugg_errs[loc] > matcher->num_errors)
+		if(sugg_list->sugg_errs[loc] > matcher->num_errors) {
 			break;
+		}
 		/* Already in the list with better score, just return */
 		if(strcmp(match,sugg_list->suggs[loc])==0) {
 			g_free(match);
@@ -676,7 +667,7 @@ static void enchant_pwl_suggest_cb(char* match,EnchantTrieMatcher* matcher)
 	}
 
 	int changes = 1;  /* num words added to list */
-
+	
 	/* Remove all elements with worse score */
 	for(size_t i=loc; i < sugg_list->n_suggs; i++){
 		g_free(sugg_list->suggs[i]);
@@ -691,8 +682,9 @@ static void enchant_pwl_suggest_cb(char* match,EnchantTrieMatcher* matcher)
 static void enchant_trie_free(EnchantTrie* trie)
 {
 	/* Don't try to free NULL or the EOSTrie pointer */
-	if(trie == NULL || trie == EOSTrie)
+	if(trie == NULL || trie == EOSTrie) {
 		return;
+	}
 
 	/* Because we have not set a destroy function for the hashtable
 	 * (to make code cleaner below), we need to explicitly free all
@@ -707,17 +699,18 @@ static void enchant_trie_free(EnchantTrie* trie)
 	g_free(trie);
 }
 
-static void enchant_trie_free_cb(void* key _GL_UNUSED,
+static void enchant_trie_free_cb(void* key _GL_UNUSED_PARAMETER,
 				 void* value,
-				 void* data _GL_UNUSED)
+				 void* data _GL_UNUSED_PARAMETER)
 {
 	enchant_trie_free((EnchantTrie*) value);
 }
 
 static EnchantTrie* enchant_trie_insert(EnchantTrie* trie,const char *const word)
 {
-	if (trie == NULL)
+	if (trie == NULL) {
 		trie = g_new0(EnchantTrie, 1);
+	}
 
 	if (trie->value == NULL) {
 		if (trie->subtries == NULL) {
@@ -739,7 +732,7 @@ static EnchantTrie* enchant_trie_insert(EnchantTrie* trie,const char *const word
 		}
 	} else {
 		/* Create new hash table for subtries, and reinsert */
-		trie->subtries = g_hash_table_new_full(g_str_hash,
+		trie->subtries = g_hash_table_new_full(g_str_hash, 
 						       g_str_equal, g_free, NULL);
 		char *tmpWord = trie->value;
 		trie->value = NULL;
@@ -759,10 +752,10 @@ static void enchant_trie_remove(EnchantTrie* trie,const char *const word)
 	if (trie->value == NULL) {
 		if (trie->subtries != NULL) {
 			/* Store multiple words in subtries */
-			if (word[0] == '\0')
+			if (word[0] == '\0') {
 				/* End-of-string is marked with special node */
 				g_hash_table_remove(trie->subtries, "");
-			else {
+			} else {
 				ssize_t nxtCh = (ssize_t)(g_utf8_next_char(word) - word);
 				char *tmpWord = g_strndup(word, nxtCh);
 				EnchantTrie *subtrie = g_hash_table_lookup(trie->subtries, tmpWord);
@@ -794,13 +787,16 @@ static void enchant_trie_remove(EnchantTrie* trie,const char *const word)
 					g_list_free(keys);
 				}
 		}
-	} else if(strcmp(trie->value, word) == 0) {
-		g_free(trie->value);
-		trie->value = NULL;
+	} else {
+		if(strcmp(trie->value, word) == 0)
+		{
+			g_free(trie->value);
+			trie->value = NULL;
+		}
 	}
 }
 
-static EnchantTrie* enchant_trie_get_subtrie(EnchantTrie* trie,
+static EnchantTrie* enchant_trie_get_subtrie(EnchantTrie* trie, 
 					     EnchantTrieMatcher* matcher,
 					     char** nxtChS)
 {
@@ -822,8 +818,9 @@ static void enchant_trie_find_matches(EnchantTrie* trie,EnchantTrieMatcher *matc
 	g_return_if_fail(matcher);
 
 	/* Can't match in the empty trie */
-	if(trie == NULL)
+	if(trie == NULL) {
 		return;
+	}
 
 	/* Bail out if over the error limits */
 	if(matcher->num_errors > matcher->max_errors){
@@ -834,10 +831,12 @@ static void enchant_trie_find_matches(EnchantTrie* trie,EnchantTrieMatcher *matc
 	if (trie == EOSTrie) {
 		size_t word_len = strlen(matcher->word);
 		int errs = matcher->num_errors;
-		if((ssize_t)word_len > matcher->word_pos)
+		if((ssize_t)word_len > matcher->word_pos) {
 			matcher->num_errors = errs + word_len - matcher->word_pos;
-		if (matcher->num_errors <= matcher->max_errors)
+		}
+		if (matcher->num_errors <= matcher->max_errors) {
 			matcher->cbfunc(g_strdup(matcher->path),matcher);
+		}
 		matcher->num_errors = errs;
 		return;
 	}
@@ -848,16 +847,21 @@ static void enchant_trie_find_matches(EnchantTrie* trie,EnchantTrieMatcher *matc
 		int errs = matcher->num_errors;
 		value = trie->value;
 		if(matcher->mode == case_insensitive)
-			value = g_utf8_strdown(value, -1);
-		matcher->num_errors = errs + edit_dist(value,
+			{
+				value = g_utf8_strdown(value, -1);
+			}
+		matcher->num_errors = errs + edit_dist(value, 
 					   &(matcher->word[matcher->word_pos]));
 		if(matcher->mode == case_insensitive)
-			g_free(value);
+			{
+				g_free(value);
+			}
 
-		if (matcher->num_errors <= matcher->max_errors)
+		if (matcher->num_errors <= matcher->max_errors) {
 			matcher->cbfunc(g_strconcat(matcher->path,
 							trie->value,NULL),
 					matcher);
+		}
 		matcher->num_errors = errs;
 		return;
 	}
@@ -902,9 +906,10 @@ static void enchant_trie_find_matches_cb(void* keyV,void* subtrieV,void* matcher
 
 	ssize_t nxtChI = (ssize_t) (g_utf8_next_char(&matcher->word[matcher->word_pos]) - matcher->word);
 
-	/* Don't handle actual matches, that's already done */
-	if (strncmp(key,&matcher->word[matcher->word_pos],nxtChI-matcher->word_pos) == 0)
+	/* Dont handle actual matches, that's already done */
+	if (strncmp(key,&matcher->word[matcher->word_pos],nxtChI-matcher->word_pos) == 0) {
 		return;
+	}
 
 	enchant_trie_matcher_pushpath(matcher,key);
 
@@ -935,7 +940,7 @@ static void enchant_trie_find_matches_cb(void* keyV,void* subtrieV,void* matcher
 	}
 
 	g_free(key2);
-
+	
 	matcher->word_pos = oldPos;
 }
 
@@ -959,7 +964,7 @@ static EnchantTrieMatcher* enchant_trie_matcher_init(const char* const word,
 	EnchantTrieMatcher* matcher = g_new(EnchantTrieMatcher,1);
 	matcher->num_errors = 0;
 	matcher->max_errors = maxerrs;
-	matcher->word = g_new0(char,len+maxerrs+2); // Ensure matcher does not overrun buffer: +2 for transpose check
+	matcher->word = g_new0(char,len+maxerrs+1); // Ensure matcher does not overrun buffer
 	strcpy(matcher->word, pattern);
 	g_free(pattern);
 	matcher->word_pos = 0;
@@ -1000,8 +1005,9 @@ static void enchant_trie_matcher_poppath(EnchantTrieMatcher* matcher,int num)
 {
 	g_return_if_fail(matcher->path_pos >= 0);
 	matcher->path_pos = matcher->path_pos - num;
-	if(matcher->path_pos < 0)
+	if(matcher->path_pos < 0) {
 		matcher->path_pos = 0;
+	}
 	matcher->path[matcher->path_pos] = '\0';
 }
 
@@ -1012,7 +1018,7 @@ static int edit_dist(const char* utf8word1, const char* utf8word2)
 	gunichar * word2 = g_utf8_to_ucs4_fast(utf8word2, -1, &len2);
 
 	int * table = g_new0(int, (len1+1)*(len2+1));
-
+	
 	/* Initialise outer rows of table */
 	for (glong i = 0; i < len1 + 1; i++)
 		table[i*(len2+1)] = i;
@@ -1024,8 +1030,9 @@ static int edit_dist(const char* utf8word1, const char* utf8word2)
 		for (glong j = 1; j < len2+1; j++) {
 			int cost = word1[i-1] != word2[j-1];
 			int v1 = table[(i-1)*(len2+1)+j] + 1;
-			if (i > 1 && j > 1 && word1[i-1] == word2[j-2] && word1[i-2] == word2[j-1])
+			if (i > 1 && j > 1 && word1[i-1] == word2[j-2] && word1[i-2] == word2[j-1]) {
 				v1 = MIN (v1, table[(i-2)*(len2+1)+(j-2)] + cost);
+			}
 
 			int v2 = table[i*(len2+1)+(j-1)] + 1;
 			int v3 = table[(i-1)*(len2+1)+(j-1)] + cost;
