@@ -18,7 +18,7 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  * THE SOFTWARE.
  */
-#include <UnitTest++.h>
+#include <UnitTest++/UnitTest++.h>
 
 #include "../unittest_enchant_providers.h"
 #include <vector>
@@ -40,8 +40,10 @@ struct DictionaryCheck_TestFixture : Provider_TestFixture
         /* FIXME: hspell does not consider non-Hebrew letters to be valid letters */
         if (_dict) {
           _provider_name = _provider->identify(_provider);
-          if (strcmp(_provider_name, "hspell") == 0)
+          if (strcmp(_provider_name, "hspell") == 0) {
+            ReleaseDictionary(_dict);
             _dict = NULL;
+          }
         }
     }
 
@@ -130,9 +132,9 @@ TEST_FIXTURE(DictionaryCheck_TestFixture,
     {
       if(AddWordToDictionary(Convert(L"fianc\x00e9" L"deleteme"))) // u00e9 = Latin small letter e with acute
       {
-          CHECK( IsWordInDictionary(Convert(L"fianc\x00e9" L"deleteme")) ); //NFC
-// FIXME: This test times out on macOS >= 10.12 with "findMisspelledWordInString timed out"
+// FIXME: These tests time out on macOS >= 10.12 with "findMisspelledWordInString timed out"
 #if !(defined(__APPLE__) && defined(__MACH__))
+          CHECK( IsWordInDictionary(Convert(L"fianc\x00e9" L"deleteme")) ); //NFC
           CHECK( IsWordInDictionary(Convert(L"fiance\x0301" L"deleteme")) ); //NFD u0301 = Combining acute accent
 #endif
       }
